#include <Models\Model.mqh>
#include <mm.mqh>
//+----------------------------------------------------------------------+
//| This model use one moving average. When the current price intersects |
//| moving average from below upwards, to be produced purchase. When the |
//| current price intersects moving average from top to down, to be      |
//| produced sale. This algoritm using simple tral-stop                  |
//+----------------------------------------------------------------------+

class cmodel_amacross : public CModel
{
  private:
    int               m_ma1;
    int               m_ma2;
    int               m_bar_tral_size;
    double            m_ma_buff_main1[];
    double            m_ma_buff_main2[];
    MqlRates          m_rates[];
    double            m_current_price;
    double            m_price_ma1;
    double            m_price_ma2;
    int               m_handle_ma1;
    int               m_handle_ma2;
    t_period          m_timing_buy;
    t_period          m_timing_sell;
  public:
    cmodel_amacross();
    bool              Init();
    bool              Init(ulong magic, string name, string symbol, ENUM_TIMEFRAMES TimeFrame, double delta, uint ma1, uint ma2, uint size_bral_tral);
    bool              Processing();
  protected:
    bool              InitIndicators();
    bool              LongOpened();
    bool              ShortOpened();
    bool              LongClosed();
    bool              ShortClosed();
    bool              TralOrders();
};

cmodel_amacross::cmodel_amacross()
{
  m_handle_ma1 = INVALID_HANDLE;
  //ArraySetAsSeries(m_ma_buff_main1, true);
  //ArraySetAsSeries(m_ma_buff_main2, true);
  //ArraySetAsSeries(m_rates, true);
  m_current_price = 0.0;
  m_price_ma1 = 0.0;
}

bool cmodel_amacross::Init()
{
  m_magic         = 142383;
  m_model_name    = "Adaptive MA Crossing";
  m_symbol        = _Symbol;
  m_timeframe     = _Period;
  m_ma1           = 12;
  m_bar_tral_size = 10;
  m_delta         = 0;
  if(!InitIndicators()) return(false);
  return(true);
}

bool cmodel_amacross::Init(ulong magic, string name, string symbol, ENUM_TIMEFRAMES TimeFrame, double delta, uint ma1, uint ma2, uint size_bral_tral)
{
  if(ma1 == 0) ma1 = 12;
  if(ma2 == 0) ma2 = 21;

  m_magic = magic;
  m_model_name = name;
  m_symbol = symbol;
  m_timeframe = TimeFrame;
  m_ma1 = ma1;
  m_ma2 = ma2;
  m_bar_tral_size = size_bral_tral;
  m_delta = delta;
  if(!InitIndicators()) return(false);
  return(true);
}

bool cmodel_amacross::InitIndicators()
{
  if(m_handle_ma1 == INVALID_HANDLE)
  {
    Print("Load indicators...");
    if((m_handle_ma1 = iAMA(m_symbol, m_timeframe, m_ma1, 2, 30, 1, PRICE_TYPICAL)) == INVALID_HANDLE)
    {
      printf("Error creating AMA1 indicator");
      return(false);
    }
    if((m_handle_ma2 = iAMA(m_symbol, m_timeframe, m_ma2, 2, 30, 1, PRICE_TYPICAL)) == INVALID_HANDLE)
    {
      printf("Error creating AMA2 indicator");
      return(false);
    }
  }
  return(true);
}

bool cmodel_amacross::Processing()
{
  m_symbol_info.Name(m_symbol);
  m_symbol_info.RefreshRates();

  if(m_symbol_info.TradeMode() == SYMBOL_TRADE_MODE_DISABLED) return(false);
  if(m_account_info.TradeAllowed() == false) return(false);
  if(m_account_info.TradeExpert() == false) return(false);

  Print(__FUNCTION__);

  TralOrders();
  GetNumberOrders(m_orders);
  if(m_orders.buy_orders > 0) LongClosed();
  else                        LongOpened();
  if(m_orders.sell_orders > 0) ShortClosed();
  else                         ShortOpened();
  return(true);
}

bool cmodel_amacross::LongOpened(void)
{
  if(m_symbol_info.TradeMode() == SYMBOL_TRADE_MODE_SHORTONLY)return(false);
  if(m_symbol_info.TradeMode() == SYMBOL_TRADE_MODE_CLOSEONLY)return(false);

  bool rezult, time_buy = false;
  double lot = 0.1;
  double sl = 0.0;
  mm open_mm;
  m_symbol_info.Name(m_symbol);
  m_symbol_info.RefreshRates();

  CopyBuffer(m_handle_ma1, 0, 0, 3, m_ma_buff_main1);
  CopyBuffer(m_handle_ma2, 0, 0, 3, m_ma_buff_main2);

  GetNumberOrders(m_orders);
  //Print("LongOpened");
  time_buy = timing(m_symbol, m_timeframe, m_timing_buy);
  // fast ama goes up
  if(/*m_ma_buff_main1[0] <= m_ma_buff_main2[0] &&*/ m_ma_buff_main1[1] > m_ma_buff_main2[1] && m_orders.buy_orders == 0 && time_buy)
  {
    //lot=open_mm.optimal_f(m_symbol, ORDER_TYPE_BUY, m_symbol_info.Ask(), 0.0, m_delta);
    lot = open_mm.jons_fp(m_symbol, ORDER_TYPE_BUY, m_symbol_info.Ask(), 0.1, 10000, m_delta);
    sl = iLowest(m_symbol, m_timeframe, MODE_LOW, 1, m_bar_tral_size);
    Print("    : ", sl);
    rezult = SendOrder(m_symbol, ORDER_TYPE_BUY, ORDER_ADD, 0, lot, m_symbol_info.Ask(), sl, 0, "AMA Buy");
    return(rezult);
  }
  return(false);
}

bool cmodel_amacross::ShortOpened(void)
{
  if(m_symbol_info.TradeMode() == SYMBOL_TRADE_MODE_LONGONLY)return(false);
  if(m_symbol_info.TradeMode() == SYMBOL_TRADE_MODE_CLOSEONLY)return(false);

  bool rezult, time_sell = false;
  double lot = 0.1;
  double sl = 0.0;
  mm open_mm;

  m_symbol_info.Name(m_symbol);
  m_symbol_info.RefreshRates();
  GetNumberOrders(m_orders);

  CopyBuffer(m_handle_ma1, 0, 0, 3, m_ma_buff_main1);
  CopyBuffer(m_handle_ma2, 0, 0, 3, m_ma_buff_main2);

  time_sell = timing(m_symbol, m_timeframe, m_timing_sell);

  // fast ama goes down
  if(/*m_ma_buff_main1[0] >= m_ma_buff_main2[0] &&*/ m_ma_buff_main1[1] < m_ma_buff_main2[1] && m_orders.sell_orders == 0 && time_sell)
  {
    //lot=open_mm.optimal_f(m_symbol, ORDER_TYPE_SELL, m_symbol_info.Bid(), 0.0, m_delta);
    lot = open_mm.jons_fp(m_symbol, ORDER_TYPE_SELL, m_symbol_info.Bid(), 0.1, 10000, m_delta);
    sl = iHighest(m_symbol, m_timeframe, MODE_HIGH, 1, m_bar_tral_size);
    rezult = SendOrder(m_symbol, ORDER_TYPE_SELL, ORDER_ADD, 0, lot, m_symbol_info.Bid(), sl, 0, "AMA Sell");
    return(rezult);
  }
  return(false);
}

bool cmodel_amacross::LongClosed(void)
{
  CTableOrders *t;
  int total_elements;
  int rez = false;
  total_elements = ListTableOrders.Total();
  if(total_elements == 0)return(false);
  for(int i = total_elements - 1; i >= 0; i--)
  {
    if(CheckPointer(ListTableOrders) == POINTER_INVALID) continue;
    t = ListTableOrders.GetNodeAtIndex(i);
    if(CheckPointer(t) == POINTER_INVALID) continue;
    if(t.Type() != ORDER_TYPE_BUY) continue;
    m_symbol_info.Refresh();
    m_symbol_info.RefreshRates();
    CopyBuffer(m_handle_ma1, 0, 0, 3, m_ma_buff_main1);
    if(m_symbol_info.Bid() <= t.StopLoss() && t.StopLoss() != 0.0)
    {
      rez = SendOrder(m_symbol, ORDER_TYPE_SELL, ORDER_DELETE, t.Ticket(), t.VolumeInitial(), m_symbol_info.Bid(), 0.0, 0.0, "MA: buy closed by stop-loss");
    }
    else if(m_symbol_info.Bid() < m_ma_buff_main1[1])
    {
      rez = SendOrder(m_symbol, ORDER_TYPE_SELL, ORDER_DELETE, t.Ticket(), t.VolumeInitial(), m_symbol_info.Bid(), 0.0, 0.0, "MA: buy closed by price drawdown");
    }
  }
  return(rez);
}

bool cmodel_amacross::ShortClosed(void)
{
  CTableOrders *t;
  int total_elements;
  int rez = false;
  total_elements = ListTableOrders.Total();
  if(total_elements == 0)return(false);
  for(int i = total_elements - 1; i >= 0; i--)
  {
    if(CheckPointer(ListTableOrders) == POINTER_INVALID) continue;
    t = ListTableOrders.GetNodeAtIndex(i);
    if(CheckPointer(t) == POINTER_INVALID) continue;
    if(t.Type() != ORDER_TYPE_SELL) continue;
    m_symbol_info.Refresh();
    m_symbol_info.RefreshRates();
    CopyBuffer(m_handle_ma1, 0, 0, 3, m_ma_buff_main1);
    if(m_symbol_info.Ask() >= t.StopLoss() && t.StopLoss() != 0.0)
    {
      rez = SendOrder(m_symbol, ORDER_TYPE_BUY, ORDER_DELETE, t.Ticket(), t.VolumeInitial(), m_symbol_info.Ask(), 0.0, 0.0, "MA: sell closed by stop-loss");
    }
    else if(m_symbol_info.Ask() > m_ma_buff_main1[1])
    {
      rez = SendOrder(m_symbol, ORDER_TYPE_BUY, ORDER_DELETE, t.Ticket(), t.VolumeInitial(), m_symbol_info.Bid(), 0.0, 0.0, "MA: sell closed by price jump");
    }
  }
  return(rez);
}

bool cmodel_amacross::TralOrders(void)
{
  int total_elements;
  int rez = false;
  double sl;
  // if new bar is started
  if(timing(m_symbol, m_timeframe, m_timing))
  {
    CopyRates(m_symbol, m_timeframe, 0, m_bar_tral_size, m_rates);
    CTableOrders *t;
    total_elements = ListTableOrders.Total();
    //Print("Total Elements: ", total_elements);
    if(total_elements == 0)return(false);
    for(int i = total_elements - 1; i >= 0; i--)
    {
      if(CheckPointer(ListTableOrders) == POINTER_INVALID)continue;
      t = ListTableOrders.GetNodeAtIndex(i);
      if(CheckPointer(t) == POINTER_INVALID)continue;
      m_symbol_info.Refresh();
      m_symbol_info.RefreshRates();
      if(t.Type() == ORDER_TYPE_BUY)
      {
        sl = iLowest(m_symbol, m_timeframe, MODE_LOW, 1, m_bar_tral_size);
        if(sl > t.StopLoss())t.StopLoss(sl);
      }
      if(t.Type() == ORDER_TYPE_SELL)
      {
        sl = iHighest(m_symbol, m_timeframe, MODE_HIGH, 1, m_bar_tral_size);
        if(sl < t.StopLoss())t.StopLoss(sl);
      }
    }
  }
  return(true);
}

